@@ -2,7 +2,7 @@ use MooseX::Declare;
 
 use 5.010;
 
-class Data::DPath {
+class Data::DPath is dirty {
 
         our $DEBUG = 0;
 
@@ -16,11 +16,18 @@ class Data::DPath {
                 };
         }
 
-        use namespace::clean -except => 'meta';
+        sub build_dpathr {
+                return sub ($) {
+                        my ($path) = @_;
+                        new Data::DPath::Path(path => $path, give_references => 1);
+                };
+        }
+
+        clean;
 
         use Sub::Exporter -setup => {
-                exports => [ dpath => \&build_dpath ],
-                groups  => { all  => [ 'dpath' ] },
+                exports => [ dpath => \&build_dpath, dpathr => \&build_dpathr ],
+                groups  => { all   => [ 'dpath', 'dpathr' ] },
         };
 
         method get_context ($class: Any $data, Str $path) {
@@ -37,7 +44,7 @@ class Data::DPath {
 
 # help the CPAN indexer
 package Data::DPath;
-our $VERSION = '0.10';
+our $VERSION = '0.12';
 
 1;
 
@@ -103,6 +110,13 @@ fitting to above data structure):
     $data ~~ dpath '//AAA/"*"[ key =~ /CC/ ]'   # when path is quoted, filter can contain slashes
     $data ~~ dpath '//CCC/*[value eq "RR2"]'    # filter by values of hashes
 
+You can get references into the C<$data> data structure by using C<dpathr>:
+
+    $data ~~ dpathr '/AAA/*/CCC'
+    $data ~~ dpathr '/AAA/BBB/CCC/../..'
+    $data ~~ dpathr '//AAA'
+    # etc.
+
 See full details C<t/data_dpath.t>.
 
 =head1 ABOUT
@@ -152,6 +166,11 @@ can omit the parens in many cases.
 
 See SYNOPSIS.
 
+=head2 dpathr( $dpath )
+
+Same as C<dpath> but toggles that results are references to the
+matched points in the data structure.
+
 =head1 METHODS
 
 =head2 match( $data, $path )
